/*
 * Copyright 2008 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 */


/*
 * Logger.hpp
 *
 *  Created on: 2010/12/13
 *      Author: HARK Development Team
 */

#ifndef LOGGER_HPP_
#define LOGGER_HPP_

#include <string>
#include <map>
#include <vector>
#include <ostream>
#include <boost/shared_ptr.hpp>
#include <boost/format.hpp>

namespace NSHarkBinaural
{

/**
 * ロギング出力のログレベルを表す.
 */
struct LogLevel
{
    /**
     * すべてのメッセージをロギングする.
     */
    static const int ALL;

    static const int FINEST = 1;
    static const int FINER = 2;
    static const int FINE = 3;
    static const int CONFIG = 4;
    static const int INFO = 5;
    static const int WARNING = 6;
    static const int SEVERE = 7;

    /**
     * ロギングをオフする.
     */
    static const int OFF;

    LogLevel(int level)
        : level(level)
    { }

    std::string ToString();

    operator int() {
        return this->level;
    }

    int level;
};

/**
 * ロギングする.
 */
class Logger
{
public:
    /**
     * グローバルロガーの名前.
     */
    static const std::string GLOBAL_LOGGER_NAME;

    /**
     * 名前で指定されるロガーを取得する.
     * GLOBAL_LOGGER_NAME以外の名前が始めて指定されると、
     * 出力ストリームが空のロガーが生成され、返される。
     */
    static Logger& GetLogger(const std::string& name);

    void AddHandler(std::ostream *os, bool needDelete = false);

    void SetLevel(LogLevel newLevel)
    {
        this->logLevel = newLevel;
    }

    LogLevel GetLevel() const
    {
        return this->logLevel;
    }

    void Log(LogLevel level, const std::string& msg, const std::string& file = "", const std::string& func = "");
    void Log(LogLevel level, const boost::format& msg, const std::string& file = "", const std::string& func = "")
    {
        Log(level, msg.str(), file, func);
    }

    void Finest(const std::string& msg, const std::string& file = "", const std::string& func = "")
    {
        Log(LogLevel::FINEST, msg, file, func);
    }
    void Finer(const std::string& msg, const std::string& file = "", const std::string& func = "")
    {
        Log(LogLevel::FINER, msg, file, func);
    }
    void Fine(const std::string& msg, const std::string& file = "", const std::string& func = "")
    {
        Log(LogLevel::FINE, msg, file, func);
    }
    void Config(const std::string& msg, const std::string& file = "", const std::string& func = "")
    {
        Log(LogLevel::CONFIG, msg, file, func);
    }
    void Info(const std::string& msg, const std::string& file = "", const std::string& func = "")
    {
        Log(LogLevel::INFO, msg, file, func);
    }
    void Warning(const std::string& msg, const std::string& file = "", const std::string& func = "")
    {
        Log(LogLevel::WARNING, msg, file, func);
    }
    void Severe(const std::string& msg, const std::string& file = "", const std::string& func = "")
    {
        Log(LogLevel::SEVERE, msg, file, func);
    }

    void Finest(const boost::format& msg, const std::string& file = "", const std::string& func = "")
    {
        Log(LogLevel::FINEST, msg, file, func);
    }
    void Finer(const boost::format& msg, const std::string& file = "", const std::string& func = "")
    {
        Log(LogLevel::FINER, msg, file, func);
    }
    void Fine(const boost::format& msg, const std::string& file = "", const std::string& func = "")
    {
        Log(LogLevel::FINE, msg, file, func);
    }
    void Config(const boost::format& msg, const std::string& file = "", const std::string& func = "")
    {
        Log(LogLevel::CONFIG, msg, file, func);
    }
    void Info(const boost::format& msg, const std::string& file = "", const std::string& func = "")
    {
        Log(LogLevel::INFO, msg, file, func);
    }
    void Warning(const boost::format& msg, const std::string& file = "", const std::string& func = "")
    {
        Log(LogLevel::WARNING, msg, file, func);
    }
    void Severe(const boost::format& msg, const std::string& file = "", const std::string& func = "")
    {
        Log(LogLevel::SEVERE, msg, file, func);
    }

    virtual ~Logger();

protected:
    Logger(const std::string& name);
    static void CreateLogger(const std::string& name);
private:
    struct OstreamPtr
    {
        std::ostream *ptr;
        bool needDelete;
    };

    LogLevel logLevel;
    const std::string name;
    std::vector<OstreamPtr> ostreams;
    static std::map<std::string, boost::shared_ptr<Logger> > loggers;
};

//#define LOG_FUNC_NAME __PRETTY_FUNCTION__
#define LOG_FUNC_NAME __func__

#define LogFinest(logger, msg) \
    do {if (logger.GetLevel() <= LogLevel::FINEST) (logger).Finest(msg, __FILE__, LOG_FUNC_NAME);} while(0)

#define LogFiner(logger, msg) \
    do {if (logger.GetLevel() <= LogLevel::FINER) (logger).Finer(msg, __FILE__, LOG_FUNC_NAME);} while(0)

#define LogFine(logger, msg) \
    do {if (logger.GetLevel() <= LogLevel::FINE) (logger).Fine(msg, __FILE__, LOG_FUNC_NAME);} while(0)
//  (logger).Fine(msg, __FILE__, LOG_FUNC_NAME)

#define LogConfig(logger, msg) \
    do {if (logger.GetLevel() <= LogLevel::CONFIG) (logger).Config(msg, __FILE__, LOG_FUNC_NAME);} while(0)
//  (logger).Config(msg, __FILE__, LOG_FUNC_NAME)

#define LogInfo(logger, msg) \
    do {if (logger.GetLevel() <= LogLevel::INFO) (logger).Info(msg, __FILE__, LOG_FUNC_NAME);} while(0)
//  (logger).Info(msg, __FILE__, LOG_FUNC_NAME)

#define LogWarning(logger, msg) \
    do {if (logger.GetLevel() <= LogLevel::WARNING) (logger).Warning(msg, __FILE__, LOG_FUNC_NAME);} while(0)
//  (logger).Warning(msg, __FILE__, LOG_FUNC_NAME)

#define LogSevere(logger, msg) \
    do {if (logger.GetLevel() <= LogLevel::SEVERE) (logger).Severe(msg, __FILE__, LOG_FUNC_NAME);} while(0)
//  (logger).Severe(msg, __FILE__, LOG_FUNC_NAME)
}

#endif /* LOGGER_HPP_ */
